// Main.java
//
// Informatics 45 Spring 2010
// Code Example: Using Libraries Other than the Java Library
//
// This is a short program that performs a Google search using their
// HTTP-based search API.  That API is similar to the Twitter API, except
// that they always return their result in a format called JSON.  This
// program makes use of a library called "JSON in Java" to parse the
// JSON and extract information from it.
//
// As with the previous web-related examples, this is a short program
// that we wrote to experiment with Google, so we made no attempt to
// handle exceptions or to organize our program well.

package inf45.spring2010.examples.web4;

import java.io.*;
import java.net.*;
import java.util.*;
import org.json.*;


public class Main
{
	public static void main(String[] args)
	throws Exception
	{
		Scanner in = new Scanner(System.in);
		
		System.out.print("What do you want to search for? ");
		String searchTerms = in.nextLine();
		
		
		// We are required to "URL encode" our search query, because it
		// is likely to contain characters that are illegal within a
		// URL (including, believe it or not, spaces).
		URL url = new URL(
			"http://ajax.googleapis.com/ajax/services/search/web?v=1.0&q="
			+ URLEncoder.encode(searchTerms, "UTF-8"));
		
		URLConnection connection = url.openConnection();

		// One of the things that Google's API requires is for us to add
		// a "Referer" header to our HTTP request.  The "Referer" header
		// is a way for web sites to know who linked to them; when you
		// navigate to a web site, your browser includes the "Referer"
		// in your HTTP request, which tells the web site where you're
		// coming from.  This allows the web site owner to analyze that
		// data to find out, for example, what search queries typically
		// lead to the site.
		//
		// Of course, since we're not running this program within a
		// browser, we're not really coming from anywhere, so we'll
		// instead list our own web address.  (In this case, I'm using
		// the Informatics 45 web site.)
		connection.addRequestProperty(
			"Referer", "http://www.ics.uci.edu/~thornton/inf45/");
		
		Scanner connectionIn = new Scanner(connection.getInputStream());

		String response = connectionIn.nextLine();


		// A JSONObject represents a JSON object, meaning that it is a set
		// of key/value pairs, where the keys are always strings and the
		// values can be one of a few different types of data: other JSON
		// objects, strings, numbers, booleans, or null.
		JSONObject json = new JSONObject(response);

		
		// This prints out the JSON with indention used to "pretty it up"
		// so we can see its structure more easily.
		System.out.println(json.toString(2));


		// If, rather than printing out the JSON string, we want to pull
		// data out of it, we can continue to use the JSON in Java library
		// to "drill down" into the data.  For example, suppose we want to
		// print out the title and URL for each of the results.  The relevant
		// part of the JSON object returned by the search query is:
		//
		//    {
		//        "responseData": {
		//            ...
		//            "results": [
		//                {
		//                    ...
		//                    "titleNoFormatting": "Informatics 45 Spring 2009",
		//                    ...
		//                    "url": "http://www.ics.uci.edu/~thornton/inf45/"
		//                    ...
		//                },
		//                ...
		//            ]
		//            ...
		//        }
		//        ...
		//    }
		//
		// So, we need to get the object whose key is "responseData", then get
		// the array within that object whose key is "results", then, for each
		// object in that array, get the "titleNoFormatting" and "url" values.

		JSONArray resultsArray =
			json.getJSONObject("responseData")
			.getJSONArray("results");
			
		for (int i = 0; i < resultsArray.length(); i++)
		{
			JSONObject resultObject = resultsArray.getJSONObject(i);
			
			System.out.println(resultObject.getString("titleNoFormatting"));
			System.out.println(resultObject.getString("url"));
			System.out.println();
		}
		

		// Finally, we close our connection, since we don't need it anymore.
		connectionIn.close();
	}
}
